using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.AnalysisServices.AdomdClient;
using System.Data;

namespace AdomdErrorHandling
{
	class Program
	{
		static void Main(string[] args)
		{
			AdomdCellErrors();
			AdomdErrorHandling();
		}
		static void AdomdCellErrors()
		{
			//Open a connection to the server
			AdomdConnection con = new AdomdConnection("Datasource=localhost; Initial Catalog=foodmart 2008;");
			con.Open();

			//Create a command that contains a calculated member with an error.
			//The Method Max accepts only numerical values as parameters, and we try to pass strings.
			AdomdCommand command = con.CreateCommand();
			command.CommandText = "WITH member measures.x AS \'Order(\"string\",4)\' select {x} ON COLUMNS FROM [Warehouse and Sales]";
			CellSet cellset = command.ExecuteCellSet();

			//Iterate all the cells and print their values
			foreach (Cell cell in cellset.Cells)
			{
				try
				{
					Console.WriteLine(cell.Value);
				}
				catch (AdomdErrorResponseException errorCell)
				{
					// If there was an error, iterate the error collection and print the error message
					Console.WriteLine(" Error in the cell ");
					foreach (AdomdError cellError in errorCell.Errors)
					{
						Console.WriteLine(cellError.Message);
					}
				}
				catch (AdomdException ex)
				{
					Console.Write(ex.Message);
				}
			}

			//Close the connection
			con.Close();
		}
		static void AdomdErrorHandling()
		{
			AdomdConnection con = new AdomdConnection("Datasource=localhost; Initial Catalog=foodmart 2000;");
			try
			{
				//Open a connection to the server
				con.Open();

				//do some operations
				//...
			}
			catch (AdomdConnectionException conException)
			{
				Console.WriteLine(conException.Message);
			}
			catch (AdomdErrorResponseException errorException)
			{
				// If there was an error, iterate the error collection and print the error message
				foreach (AdomdError cellError in errorException.Errors)
				{
					Console.WriteLine(errorException.Message);
				}
			}
			catch (AdomdUnknownResponseException ex)
			{
				Console.WriteLine(ex.Message);
			}
			catch (AdomdCacheExpiredException cacheExeption)
			{
				Console.WriteLine(cacheExeption.Message);
				//refresh the controls
				//...
			}
			finally
			{
				//Close the connection
				if ( con.State == ConnectionState.Open )
				   con.Close();
			}
		}
	}
}
