using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.AnalysisServices.AdomdClient;
using System.Data;
using System.Xml;

namespace AdomdExecuteSample
{
	class Program
	{
		static void Main(string[] args)
		{
			AdomdExecuteCommandText();
			AdomdExecuteCommandStream();
			AdomdExecuteCommandCellSet();
			AdomdExecuteCommandDataReader();
			AdomdExecuteCommandXmlReader();
			AdomdLoadCellSet();
			AdomdExecuteCommandNonQuery();
			AdomdExecuteCommand();
		}

		static void AdomdExecuteCommandText()
		{
			//Open a connection to the server
			String conString = "Datasource=localhost; Initial Catalog=Foodmart 2008";
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = conString;
			con.Open();

			// Create the object AdomdCommand
			AdomdCommand command = new AdomdCommand();
			command.Connection = con;

			//Define the text the request
			command.CommandText = "SELECT Measures.Members ON COLUMNS FROM [Warehouse and Sales]";

			//Execute command...
			//
			// Close the connection
			con.Close();
		}
		static void AdomdExecuteCommandStream()
		{
			//Open a connection to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost;";
			con.Open();

			//Open a stream for reading a file that contains the XML for creating a cube
			System.IO.StreamReader sr = new System.IO.StreamReader("..\\..\\Foodmart2008Metadata.xml", true);
			AdomdCommand command = con.CreateCommand();
			command.CommandStream = sr.BaseStream;

			// Execute the command
			command.ExecuteNonQuery();

			// Close the stream
			sr.Close();
		}
		static void AdomdExecuteCommandCellSet()
		{
			// Connect to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008";
			con.Open();

			//Create a simple MDX request
			String cmdText = "SELECT Measures.Members ON COLUMNS FROM [Warehouse and Sales]";
			AdomdCommand command = new AdomdCommand(cmdText,con);

			//Execute a method, that returns CellSet
			CellSet cellset = command.ExecuteCellSet();

			//Close the connection
			con.Close();
		}
		static void AdomdExecuteCommandDataReader()
		{
		// Connect to the server
		AdomdConnection con = new AdomdConnection();
		con.ConnectionString = "Datasource = localhost; Initial Catalog=Foodmart 2008";
		con.Open();

		// Create a simple MDX statement
		AdomdCommand command = (AdomdCommand)con.CreateCommand();
		string query = "SELECT Measures.Members ON columns FROM [Warehouse and Sales]";
		command.CommandText = query;

		//Execute the method that returns IDataReader
		IDataReader reader = command.ExecuteReader();

		// Read the metadata about the columns in the returned result
		DataTable schemaTable = reader.GetSchemaTable();
		Console.WriteLine("Schema");
		for(int iSchRow = 0; iSchRow<schemaTable.Rows.Count; iSchRow ++)
		{
		for(int iSchCol = 0; iSchCol<schemaTable.Columns.Count; iSchCol ++)
			{
			Console.WriteLine(schemaTable.Rows[iSchRow][iSchCol].ToString() + " ");
			}
		}
		// Read the result
		while(reader.Read())
		{
			for(int iCol = 0; iCol <reader.FieldCount; iCol ++)
			{
				Console.WriteLine(reader.GetValue(iCol).ToString());
				Console.WriteLine(reader.GetDataTypeName(iCol));
			}
		}
		// Close DataReader
		reader.Close();
		// Close the connection
		con.Close();
		}
		static void AdomdExecuteCommandXmlReader()
		{
			// Open a connection to a source of data
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008;";
			con.Open();

			// Create a command
			AdomdCommand command = con.CreateCommand();
			command.CommandText = "SELECT Measures.Members ON COLUMNS FROM [Warehouse and Sales]";

			// Execute the command and get XMLReader
			XmlReader reader = command.ExecuteXmlReader();

			// Write the XML into a file
			string xml = reader.ReadOuterXml();
			System.IO.StreamWriter writer = new System.IO.StreamWriter("cellSetXml.xml", false);
			writer.Write(xml);
			// Close the file
			writer.Close();
			// Close XmlReader
			reader.Close();
			// Close the connection
			con.Close();
		}
		static void AdomdLoadCellSet()
		{
			// Open XmlReader from the XML file
			XmlTextReader reader = new XmlTextReader("cellSetXml.xml");
			// Load XML into CellSet object
			CellSet cellSet = CellSet.LoadXml(reader);

			// Close XmlReader 
			reader.Close();
		}
		static void AdomdExecuteCommandNonQuery()
		{
			// Connect to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008";
			con.Open();

			// Create the command that will send a request for a creation of a calculated member
			String cmdText = "CREATE member [Warehouse and Sales].Measures.MyCalcMember as '1' ";
			AdomdCommand command = new AdomdCommand(cmdText, con);

			// Execute the command, without expecting a returned result
			command.ExecuteNonQuery();

			//Close the connection
			con.Close();
		}
		static void AdomdExecuteCommand()
		{
			//Connect to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008";
			con.Open();

			//Create a command that sends a simple MDX request
			String cmdText = "SELECT Measures.Members ON COLUMNS FROM [Warehouse and Sales]";
			AdomdCommand command = new AdomdCommand(cmdText, con);

			//Execute a command and return an object of any type.
			Object result = command.Execute();

			//Analyze the type of object returned 
			if (null == result)
			{
				//No result
			}
			else if (result is CellSet)
			{
				//Work with CellSet
				CellSet cellset = (CellSet)result;
			}
			else if (result is AdomdDataReader)
			{
				//Work with DataReader
				AdomdDataReader dataReader = (AdomdDataReader)result;
			}
			//Close the connection
			con.Close();
		}
	}
}
