using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.AnalysisServices.AdomdClient;

namespace AdomdGetMembers
{
	class Program
	{
		static void Main(string[] args)
		{
			AdomdGetMembers();
			AdomdGetMembersWithStartAndCount();
			AdomdGetMembersWithFilter();
			AdomdGetMembersWithProperties();

		}

		static void AdomdGetMembers()
		{
			//Open a connection to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008;";
			con.Open();

			//Go to the level
			Level lvl = con.Cubes["Warehouse and Sales"].Dimensions["Store"].Hierarchies["Stores"].Levels ["Store"];

			//Receive a collection of members of the level 
			MemberCollection allMembers = lvl.GetMembers();

			//Print all the elements of a collection
			foreach(Member member in allMembers)
			{
				Console.WriteLine(member.UniqueName + ";");
			}
			// Verify that the MemberProperty collection  is empty, despite existing custom member properties at each element at the level Store Name
			Member firstMember = allMembers[0];
			foreach(MemberProperty memberProperty in firstMember.MemberProperties)
			{
				Console.WriteLine(memberProperty.UniqueName);
			}

			//Close the connection
			con.Close();
		}
		static void AdomdGetMembersWithStartAndCount()
		{
			//Set an initial element of the member and number of returned elements
			long start = 3;
			long count = 10;

			//Open a connection to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008; ";
			con.Open();

			//Go to the level.
			Level lvl = con.Cubes["Warehouse and Sales"].Dimensions["Store"].Hierarchies["Stores"].Levels["Store"];

			//Receive a collection of the members of the level, starting with member number 3 and containing 10 members
			MemberCollection allMembers = lvl.GetMembers(start, count);

			//Print all the elements of the collection
			foreach(Member member in allMembers)
			{
				Console.WriteLine(member.UniqueName + ";");
			}
			//Verify that the collection MemberProperty is empty, despite the existence of custom member properties at each member at the level Store
			Member firstMember = allMembers[0];
			foreach(MemberProperty memberProperty in firstMember.MemberProperties)
			{
				Console.WriteLine(memberProperty.UniqueName);
			}

			//Close the connection
			con.Close();
	    }
		static void AdomdGetMembersWithFilter()
		{
			//Open a connection to the server
			AdomdConnection con = new AdomdConnection();
			con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008;";
			con.Open();

			//Get the level
			Level lvl = con.Cubes["Warehouse and Sales"].Dimensions["Store"].Hierarchies["Stores"].Levels["Store"];

			// Set restrictions for the size of the collection. 
			// In this case, we want to get all the elements that satisfy the filter
			long start = 0;
			long count = lvl.MemberCount;

			//Create the filter object.
			// Choose the elements of the dimension member property Store Manager that begin with the letter R
			MemberFilter memberFilter = new MemberFilter(
			"Store Manager",
			MemberFilterType.BeginsWith, 
				"R");

			//Request members
			MemberCollection allMembers = lvl.GetMembers(start, count, memberFilter);

			// Print all the elements of the collection
			foreach(Member member in allMembers)
			{
				Console.WriteLine(member.UniqueName + ";");
			}

			// Close the connection 
			con.Close();
		}
		static void AdomdGetMembersWithProperties()
		{
			try
			{
				//Open a connection to a source of data
				AdomdConnection con = new AdomdConnection();
				con.ConnectionString = "Datasource=localhost; Initial Catalog=Foodmart 2008;";
				con.Open();

				//Get the level
				Level lvl = con.Cubes["Warehouse and Sales"].Dimensions["Store"].Hierarchies["Stores"].Levels["Store"];
				long start = 0;
				long count = lvl.MemberCount;
				MemberFilter mf = new MemberFilter("Store Manager", MemberFilterType.BeginsWith, "R");

				//Create an array of member properties 
				string[] properties = new string[lvl.LevelProperties.Count];
				int i = 0;

				//Iterate MemberProperties and store the unique names of member properties
				foreach (LevelProperty levelProperty in lvl.LevelProperties)
				{
					properties[i++] = levelProperty.UniqueName;
				}

				//Get the members and the member properties that are interesting to us
				MemberCollection allMembers = lvl.GetMembers(start, count, properties, mf);

				//Print the list of members
				foreach (Member member in allMembers)
				{
					Console.WriteLine(member.UniqueName + ";");
				}
				//If there is at least one member in the collection, print its member property
				if (allMembers.Count > 0)
				{
					Member firstMember = allMembers[0];
					foreach (MemberProperty memberProperty in firstMember.MemberProperties)
					{
						Console.WriteLine(memberProperty.UniqueName + "=" + memberProperty.Value);
					}
				}
				//Close the connection
				con.Close();
			}
			catch (Exception ex)
			{
				Console.WriteLine(ex.Message);
			}
		}

	}
}
