using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.AnalysisServices.AdomdClient;

namespace AdomdIterateOlapInfo
{
	class Program
	{
		static void Main(string[] args)
		{
			// Open a connection to the server
			AdomdConnection con = new AdomdConnection("Datasource=localhost; Initial Catalog=Foodmart 2008;");
			con.Open();

			//Send a simple MDX statement
			AdomdCommand command = con.CreateCommand();
			command.CommandText = "SELECT Measures.Members ON COLUMNS FROM [Warehouse and Sales]";
			CellSet cellset = command.ExecuteCellSet();

			// Iterate the data on the cubes used in the request
			// In the current version Microsoft SQL Server Analysis Services, only one cube can be used in the request
			foreach (OlapInfoCube cubeInfo in cellset.OlapInfo.CubeInfo.Cubes)
			{
				Console.WriteLine("CubeName = " + cubeInfo.CubeName);
				Console.WriteLine("Cube was last updated on " + cubeInfo.LastDataUpdate);
			}

			// Iterate the metadata for the axes 
			foreach (OlapInfoAxis axisInfo in cellset.OlapInfo.AxesInfo.Axes)
			{
				// For each axis, print the name of the axis
				Console.WriteLine("AxisName = " + axisInfo.Name);
				//Iterate the hierarchies on the current axis.
				foreach (OlapInfoHierarchy hierarchyInfo in axisInfo.Hierarchies)
				{
					Console.WriteLine("HierarchyName = " + hierarchyInfo.Name);
					foreach (OlapInfoProperty hierarchyPropertyInfo in hierarchyInfo.HierarchyProperties)
					{
						Console.WriteLine("HierarchyProperty = " + hierarchyPropertyInfo.Name);
					}
				}
			}

			// Iterate the cell properties that are  used in the request
			foreach (OlapInfoProperty cellPropertyInfo in cellset.OlapInfo.CellInfo.CellProperties)
			{
				Console.WriteLine("CellPropertyName = " + cellPropertyInfo.Name);
			}

			//Close the connection
			con.Close();
		}
	}
}
