﻿#region Using directives

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

#endregion
using Microsoft.AnalysisServices.AdomdClient;

namespace QueryCancel
{
	partial class QueryCancelForm : Form
	{
		AdomdConnection connection = null;
		AdomdCommand command = null;
		BackgroundWorker backgroundWorker = null;

		public QueryCancelForm()
		{
			InitializeComponent();
			cancelButton.Enabled = false;
			executeButton.Enabled = false;
			connectionButton.Enabled = true;

			this.backgroundWorker = new BackgroundWorker();
			this.backgroundWorker.WorkerReportsProgress = false;
			this.backgroundWorker.WorkerSupportsCancellation = true;

			//In C# 2.0 you can use just the method name, and the       //compiler will infer the delegate type
			//backgroundWorker.ProgressChanged += OnProgressChanged;
			this.backgroundWorker.RunWorkerCompleted += OnCompleted;
			this.backgroundWorker.DoWork += OnDoWork;
		}

		private void connectionButton_Click(object sender, EventArgs e)
		{
			this.connection = new AdomdConnection();
			this.connection.ConnectionString = "Datasource=localhost;Initial Catalog=Foodmart 2008;";
			this.connection.Open();

			cancelButton.Enabled = false;
			executeButton.Enabled = true;
			connectionButton.Enabled = false;
		}

		private void executeButton_Click(object sender, EventArgs e)
		{
			executeButton.Enabled = false;

			this.command = this.connection.CreateCommand();
            this.command.CommandText = "SELECT NON EMPTY [Customer].[Customers].[Customer].Members ON 0 FROM [Warehouse and Sales]";
			cancelButton.Enabled = true;
			statusLabel.Text = "Running";

			this.backgroundWorker.RunWorkerAsync(this.command);
		}

		/// <summary>
		/// This function is executed in the working thread
		/// </summary>
		void OnDoWork(object sender, DoWorkEventArgs doWorkArgs)
		{
			BackgroundWorker backgroundWorker;
			backgroundWorker = sender as BackgroundWorker;

			AdomdCommand commandOnThread = (AdomdCommand)doWorkArgs.Argument;

			CellSet cellSet = commandOnThread.ExecuteCellSet();
			if(backgroundWorker.CancellationPending)
				doWorkArgs.Cancel = true;
			else
 				doWorkArgs.Result = cellSet;

		}
		void OnCompleted(object sender,  RunWorkerCompletedEventArgs completedArgs)
		{
			if (completedArgs.Error != null)
			{
                statusLabel.Text = "Error Occured";
                MessageBox.Show(completedArgs.Error.Message, completedArgs.Error.Source);
			}
			else if (completedArgs.Cancelled)
			{
                statusLabel.Text = "Cancelled"; 
                MessageBox.Show("Command has been canceled by user");
			}
			else
			{
                //The command completed successfully.
                CellSet cellSetResult = (CellSet)completedArgs.Result;
				foreach (Tuple tuple in cellSetResult.Axes[0].Set.Tuples)
				{
					TupleBox1.Items.Add(tuple.Members[0].Caption);
				}
                statusLabel.Text = "Completed";
            }
			if (null != this.connection && this.connection.State == ConnectionState.Open)
				this.connection.Close();
			this.connection = null;
			connectionButton.Enabled = true;
			cancelButton.Enabled = false;
			executeButton.Enabled = false;
		}

		private void cancelButton_Click(object sender, EventArgs e)
		{
            // Call the Cancel method, which sends a cancel request to the server.
            try
            {
                this.command.Cancel();
            }
            catch (AdomdErrorResponseException ex)
            {
                //If you send the Cancel command twice Analysis Services 
                //might return an error, that command is already cancelled. 
                //We assume that Cancel command can't through other exceptions.
            }
            this.backgroundWorker.CancelAsync();

		}

        private void QueryCancelForm_Load(object sender, EventArgs e)
        {

        }
	}
}